#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include "../../driver/slm.h"

using namespace std;

#define SLM			"/dev/slm"
#define DEFAULTS_FILENAME	"/etc/slm/defaults"


#define	BOOL int
#define	TRUE	1
#define	FALSE	0

BOOL	sc(char *, char *);
void	h_err(int);
void	set_lasera(int, unsigned int, unsigned int);
void	set_laserb(int, unsigned int, unsigned int);
void	save_defaults(float, float, float, float, float, float, float, float, float, float, unsigned int, unsigned int, unsigned int, unsigned int);
int	load_defaults(float *, float *, float *, float *, float *, float *, float *, float *, float *, float *, unsigned int*, unsigned int*, unsigned int*, unsigned int*);

main(int argc,char ** argv){

int		dev;
int		index=1;
BOOL		verbose_mode=FALSE;
BOOL		save_the_defaults=TRUE;
BOOL		got_lasera=FALSE;
BOOL		got_laserb=FALSE;
BOOL		got_time_period=FALSE;

float		w,f,xoff,msr,dw,df,dx,slm_height,distortion_factor; 	//don't actually need all these, just there because we use the same defaults file as
									//manual_bns, so we need to retain these values when we save the file
float		time_period,orig_time_period;
unsigned int	lasera_amp,orig_lasera_amp;
unsigned int	laserb_amp,orig_laserb_amp;
unsigned int	lasera_time,orig_lasera_time;
unsigned int	laserb_time,orig_laserb_time;

	if(argc==1){
		printf("Need some arguments!\n\n");
		printf("-aa  -a_amp       : Laser A amplitude (0->255)\n");
		printf("-at  -a_time      : Laser A time      (0->65535)\n");
		printf("-ba  -b_amp       : Laser B amplitude (0->255)\n");
		printf("-bt  -b_time      : Laser B time      (0->65535)\n");
		printf("-t   -time_period : time period in secs (not set by default)\n");
		printf("-nd  -nodefaults  : Do NOT save changes in defaults file\n");
		printf("-v   -verbose     : confirm what's been done\n");
		exit(3);
		}

	if(load_defaults(&w, &f, &xoff, &msr, &dw, &df, &dx, &slm_height, &distortion_factor, &orig_time_period, &orig_lasera_amp, &orig_laserb_amp, &orig_lasera_time, &orig_laserb_time)==TRUE){
		cout<<"ERROR! Could not load the defaults file, "<<DEFAULTS_FILENAME<<endl<<"Quitting..."<<endl;
		exit(1);
		}

	lasera_amp=orig_lasera_amp;
	laserb_amp=orig_laserb_amp;
	lasera_time=orig_lasera_time;
	laserb_time=orig_laserb_time;
	time_period=orig_time_period;

	while(index<argc){
		if(sc(argv[index],"-v")||sc(argv[index],"-verbose")){
			verbose_mode=TRUE;
			}
		if(sc(argv[index],"-nd")||sc(argv[index],"-nodefaults")){
			save_the_defaults=FALSE;
			}
		if(sc(argv[index],"-aa")||sc(argv[index],"-a_amp")){
			sscanf(argv[++index],"%u",&lasera_amp);
			got_lasera=TRUE;
			}
		if(sc(argv[index],"-at")||sc(argv[index],"-a_time")){
			sscanf(argv[++index],"%u",&lasera_time);
			got_lasera=TRUE;
			}
		if(sc(argv[index],"-ba")||sc(argv[index],"-b_amp")){
			sscanf(argv[++index],"%u",&laserb_amp);
			got_laserb=TRUE;
			}
		if(sc(argv[index],"-bt")||sc(argv[index],"-b_time")){
			sscanf(argv[++index],"%u",&laserb_time);
			got_laserb=TRUE;
			}
		if(sc(argv[index],"-t")||sc(argv[index],"-time_period")){
			sscanf(argv[++index],"%f",&time_period);
			got_time_period=TRUE;
			}
		index++;
		}

	if(lasera_amp<0 || lasera_amp>255){cout<<"ERROR! Laser A amp should be >=0 and <=255! Quitting..."<<endl;exit(4);}
	if(laserb_amp<0 || laserb_amp>255){cout<<"ERROR! Laser B amp should be >=0 and <=255! Quitting..."<<endl;exit(4);}
	if(lasera_time<0 || lasera_time>65535){cout<<"ERROR! Laser A time should be >=0 and <=65535! Quitting..."<<endl;exit(4);}
	if(laserb_time<0 || laserb_time>65535){cout<<"ERROR! Laser B time should be >=0 and <=65535! Quitting..."<<endl;exit(4);}
	if((int)(time_period/164.4e-6)<6 || time_period>10){cout<<"ERROR! Time period should be >=0.001 and <=10! Quitting..."<<endl;exit(4);}

	if(verbose_mode==TRUE){
		cout<<"\nbns_laser_ab: verbose mode selected"<<endl<<endl;
		cout<<"Defaults loaded...:\n";
		cout<<"Time period: "<<orig_time_period<<endl;
		cout<<"Laser A amp: "<<orig_lasera_amp<<"   Laser A time: "<<orig_lasera_time<<endl;
		cout<<"Laser B amp: "<<orig_laserb_amp<<"   Laser B time: "<<orig_laserb_time<<endl<<endl;
		cout<<"New values...:\n";
		if(got_time_period==TRUE) cout<<"Time period: "<<time_period<<endl;
		if(got_lasera==TRUE) cout<<"Laser A amp: "<<lasera_amp<<"   Laser A time: "<<lasera_time<<endl;
		if(got_laserb==TRUE) cout<<"Laser B amp: "<<laserb_amp<<"   Laser B time: "<<laserb_time<<endl;
		if(save_the_defaults==FALSE) cout<<"These new values will NOT be saved in the defaults file!"<<endl;
		}

	dev=open(SLM,O_RDWR);
	if(dev<0){
		cout<<"Failed to open "<<SLM<<endl;
		perror("slm_test:FTO:");
		exit(1);
		}

	if(got_time_period==TRUE) h_err(ioctl(dev,BNS_SET_PERIODS,(int)(time_period/164.4e-6)));
	if(got_lasera==TRUE) set_lasera(dev,lasera_time,lasera_amp);
	if(got_laserb==TRUE) set_laserb(dev,laserb_time,laserb_amp);

//	h_err(ioctl(dev,BNS_DISPLAY_IMAGE,0));

	if(save_the_defaults==TRUE){
		if(verbose_mode==TRUE) cout<<"Saving new defaults."<<endl;
		save_defaults(w, f, xoff, msr, dw, df, dx, slm_height, distortion_factor, time_period, lasera_amp, laserb_amp, lasera_time, laserb_time);
		}
	}

void set_lasera(int dev, unsigned int t_period, unsigned int amplitude){
	div_t l_div;
	l_div=div((int)t_period,16);
	h_err(ioctl(dev,BNS_SET_LASERA,(unsigned int)(65536*l_div.quot)+(unsigned int)(256*l_div.rem)+(unsigned int)amplitude));
	}

void set_laserb(int dev, unsigned int t_period, unsigned int amplitude){
	div_t l_div;
	l_div=div((int)t_period,16);
	h_err(ioctl(dev,BNS_SET_LASERB,(unsigned int)(65536*l_div.quot)+(unsigned int)(256*l_div.rem)+(unsigned int)amplitude));
	}

void h_err(int err){
	if(err!=0){
		printf("Got error %d\n",err);
		exit(1);
		}
	usleep(1000);
	}

BOOL sc(char *con,char *var){
	if(strcmp(con,var)==0){
		return TRUE;
		}
	return FALSE;
	}

void save_defaults(float w, float f, float x, float msr, float dw, float df, float dx, float h, float dd, float t, unsigned int lasera_amp, unsigned int laserb_amp, unsigned int lasera_time, unsigned int laserb_time){

ofstream os;
        os.open(DEFAULTS_FILENAME);
        os<<w<<"\n";
        os<<f<<"\n";
        os<<x<<"\n";
        os<<msr<<"\n";
        os<<dw<<"\n";
        os<<df<<"\n";
        os<<dx<<"\n";
        os<<h<<"\n";
        os<<dd<<"\n";
        os<<t<<"\n";
        os<<lasera_amp<<"\n";
        os<<laserb_amp<<"\n";
        os<<lasera_time<<"\n";
        os<<laserb_time<<"\n";
        os.close();
        }
/* sanity extension by matt august 2002 */
int load_defaults(float *w, float *f, float *x, float *msr, float *dw, float *df, float *dx, float *h, float *dd, float *t, unsigned int *lasera_amp, unsigned int *laserb_amp, unsigned int *lasera_time, unsigned int *laserb_time){

ifstream is;

        if(access(DEFAULTS_FILENAME,R_OK)!=0){
                cout<<"*** Can't read the last settings from "<<DEFAULTS_FILENAME<<"\n";
                return TRUE;
                }
        is.open(DEFAULTS_FILENAME);
        is>>*w;
        is>>*f;
        is>>*x;
        is>>*msr;
        is>>*dw;
        is>>*df;
        is>>*dx;
        is>>*h;
        is>>*dd;
        is>>*t;
        is>>*lasera_amp;
        is>>*laserb_amp;
        is>>*lasera_time;
        is>>*laserb_time;
        is.close();
        return FALSE;
        }

