/* manual_bns adapted from "test_slm.c" (for bns slm) and "manual_slm.cc" (for displaytech slm) */
/* sds 29th Jan 2002 */

#include "manual_bns_keys.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include "../../driver/slm.h"

#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <curses.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
using namespace std;

#define SLM "/dev/slm"
#define NFRAMES 1024
#define ROW_SIZE (512)
#define IMAGE_LENGTH 512
#define SIZE 512*IMAGE_LENGTH
#define N 100000
#define DEFAULTS_FILENAME	"/etc/slm/defaults"

void h_err(int);
void set_lasera(int, unsigned int, unsigned int);
void set_laserb(int, unsigned int, unsigned int);
void SLM_upload_arcs(int,float,float,float,float,float,float);
void save_defaults(float, float, float, float, float, float, float, float, float, float, unsigned int, unsigned int, unsigned int, unsigned int);
int load_defaults(float *, float *, float *, float *, float *, float *, float *, float *, float *, float *, unsigned int*, unsigned int*, unsigned int*, unsigned int*);

main(int argc,char ** argv){

// set initial values

int*	dummy=NULL;
int	dev;
unsigned char	image[SIZE];
int n=0,err;

// NOTE IF YOU WANT TO CHANGE THESE DEFAULT VALUES, YOU MUST ALSO DO SO
// IN THE "case: DEFAULT" SECTION BELOW.....

float xoff=0;
float f=2e-3;
float w=36.9e-6;
//float slm_height=2.45e-3;	// pre 23 July 2002
//float slm_height=2.6e-3;	// 24 July 2002 -> 13 September 2002
float slm_height=2.42e-3;	// 13 September 2002 ->
float distortion_factor=1;
float msr=0.5;
float period=2; //in seconds;
unsigned int lasera_amp=255;
unsigned int laserb_amp=255;
unsigned int lasera_time=4;
unsigned int laserb_time=4;


float tmpvalue;

div_t l_div;

float dx=100e-6;
float df=100e-6;
float dw=1e-6;

int quit=FALSE;
int power_on=FALSE;
int update_slm=FALSE;
int first_run=TRUE;
char keypress;
char kp2;
char inptvalue[12];
float px,py;

	if(argc!=1){
		cout<<"No arguments!\n";
		exit(3);
		}
	initscr();
	cbreak();
	noecho();
	echo();
	noecho();
	nonl();
	intrflush(stdscr, FALSE);
	refresh();

//	cout<<"CAUTION: This program uses matt's SLM_upload arcs\r\n";
//	cout<<"As of June 2001 X corresponds to vertical (+ve goes down on sample)\r\n";
//	cout<<"and Y corresponds to horizontal (+ve goes right on sample)\r\n";
//	cout<<"The Y direction is squished by the optical beamsteer tilt\r\n";
//	cout<<"manual_slm uses (x,y)->(xoff,F)->(Y,X) hence +F goes down\r\n";
//	cout<<"+ve xoffset goes right\r\n\n";

	cout<<"Matt / Steve's manual arc manouverer for the BNS slm.\r\n\n"<<flush;
	cout<<"Load / Save defaults added matt august 2002\r\n";

	// initialisation...
        cout<<"Opening slm "<<SLM<<"\r\n"<<flush;
        dev=open(SLM,O_RDWR);
        if(dev<0){
		nocbreak(); echo(); endwin();
		cout<<"Failed to open "<<SLM<<endl;
                perror("slm_test:FTO:");
                exit(1);
                }

	cout<<"\nLoading defaults...\r\n";
	first_run=load_defaults(&w, &f, &xoff, &msr, &dw, &df, &dx, &slm_height, &distortion_factor, &period, &lasera_amp, &laserb_amp, &lasera_time, &laserb_time);

	py=slm_height/ROW_SIZE; //in metres
	px=py/distortion_factor;

	if(first_run==FALSE){
		// ie the values are set - pass them to the slm
		cout<<"loaded last values - sending to SLM\r\n";

		SLM_upload_arcs(dev,-xoff,f,py,px,w,msr);
		update_slm=FALSE;
		first_run=FALSE;
		cout<<"Ok- defaults restored\r\n";
		}

	cout<<"Setting period to "<<period<<" seconds.\r\n\n"<<flush;
        h_err(ioctl(dev,BNS_SET_PERIODS,(int)(period/164.4e-6)));
	cout<<"Setting Laser A and Laser B\r"<<endl;
	set_lasera(dev,lasera_time,lasera_amp);
	set_laserb(dev,laserb_time,laserb_amp);
	cout<<"Press `h' for help, or `q' for quit, `a' to show the arc attributes.\r\n"<<flush;

	do{
		keypress=getchar();
		switch(keypress){
			case POWER_ON:
				cout<<"Setting SLM power ON\r"<<endl;
				power_on=TRUE;
				h_err(ioctl(dev,BNS_POWER_ON,dummy));
				break;

			case POWER_OFF:
				cout<<"Setting SLM power OFF\r"<<endl;
				power_on=FALSE;
				h_err(ioctl(dev,BNS_POWER_OFF,dummy));
				break;

			case LINE_SPACING_PLUS:
				w+=dw;
				cout<<"w: "<<w*1e6<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case LINE_SPACING_MINUS:
				w-=dw;
				cout<<"w: "<<w*1e6<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case FOCAL_LENGTH_PLUS:
				f+=df;
				cout<<"Mean focal length: "<<f*1e6<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case FOCAL_LENGTH_MINUS:
				f-=df;
				cout<<"Mean focal length: "<<f*1e6<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case X_OFFSET_PLUS:
				xoff+=dx;
				cout<<"x offset: "<<xoff*1e6<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case X_OFFSET_MINUS:
				xoff-=dx;
				cout<<"x offset: "<<xoff*1e6<<" microns\r"<<endl;
				update_slm=TRUE;
				break;


			case SET_TO:
				cout<<"Set "<<flush;
				kp2=getchar();
				switch(kp2){
					case LINE_SPACING: //kept for backwards 'human compatability'
						cout<<"w.\r\nCurrent w: "<<w*1e6<<" microns. Enter new w: "<<flush;
						echo();
						getnstr(inptvalue,10);w=strtod(inptvalue,NULL)/1e6;
						noecho();
						break;
					case WAVELENGTH: //exactly the same thing as above, just a different keypress
						cout<<"w.\r\nCurrent w: "<<w*1e6<<" microns. Enter new w: "<<flush;
						echo();
						getnstr(inptvalue,10);w=strtod(inptvalue,NULL)/1e6;
						noecho();
						break;
					case FOCAL_LENGTH:
						cout<<"mean focal length.\r\nCurrent mean focal length: "<<f*1e6<<" microns . Enter new mean focal length: "<<flush;
						echo();
						getnstr(inptvalue,10);f=strtod(inptvalue,NULL)/1e6;
						cout<<"\r"<<flush;
						noecho();
						break;
					case X_OFFSET:
						cout<<"x offset.\r\nCurrent x offset: "<<xoff*1e6<<" microns. Enter new x offset: "<<flush;
						echo();
						getnstr(inptvalue,10);xoff=strtod(inptvalue,NULL)/1e6;
						cout<<"\r"<<flush;
						noecho();
						break;
					case SLM_SIZE:
						cout<<"image size.\r\nCurrent image size: "<<slm_height*1e6<<" microns. Enter new image size: "<<flush;
						echo();
						getnstr(inptvalue,10);slm_height=strtod(inptvalue,NULL)/1e6;
						cout<<"\r"<<flush;
						noecho();
						break;
					case DISTORTION_FACTOR:
						cout<<"distortion factor.\r\nCurrent distortion factor: "<<distortion_factor<<". Enter distortion factor: "<<flush;
						echo();
						getnstr(inptvalue,10);distortion_factor=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						break;
					case LASER_A_GAIN:
						cout<<"laser A amplitude.\r\nCurrent laser A amplitude: "<<lasera_amp<<". Enter new laser A amplitude: "<<flush;
						echo();
						getnstr(inptvalue,10);lasera_amp=strtoul(inptvalue,NULL,0);
						cout<<"\r"<<flush;
						noecho();
						if(lasera_amp<0 || lasera_amp>255){
							cout<<"Laser A amplitude error\n";
							}
						else{
							set_lasera(dev,lasera_time,lasera_amp);
							}
						break;	
					case LASER_B_GAIN:
						cout<<"laser B amplitude.\r\nCurrent laser B amplitude: "<<laserb_amp<<". Enter new laser B amplitude: "<<flush;
						echo();
						getnstr(inptvalue,10);laserb_amp=strtoul(inptvalue,NULL,0);
						cout<<"\r"<<flush;
						noecho();
						if(laserb_amp<0 || laserb_amp>255){
							cout<<"Laser B amplitude error\n";
							}
						else{
							set_laserb(dev,laserb_time,laserb_amp);
							}
						break;	
					case LASER_A_TIME:
						cout<<"laser A time.\r\nCurrent laser A time: "<<lasera_time<<". Enter new laser A time: "<<flush;
						echo();
						getnstr(inptvalue,10);lasera_time=strtoul(inptvalue,NULL,0);
						cout<<"\r"<<flush;
						noecho();
						if(lasera_time<0 || lasera_time>65535){
							cout<<"Laser A time error\n";
							}
						else{
							set_lasera(dev,lasera_time,lasera_amp);
							}
						break;	
					case LASER_B_TIME:
						cout<<"laser B time.\r\nCurrent laser B time: "<<laserb_time<<". Enter new laser B time: "<<flush;
						echo();
						getnstr(inptvalue,10);laserb_time=strtoul(inptvalue,NULL,0);
						cout<<"\r"<<flush;
						noecho();
						if(laserb_time<0 || laserb_time>65535){
							cout<<"Laser B time error\n";
							}
						else{
							set_laserb(dev,laserb_time,laserb_amp);
							}
						break;	
					case TIME_PERIOD:
						cout<<"time period.\r\nCurrent time period: "<<period<<" seconds. Enter new time period: "<<flush;
						echo();
						getnstr(inptvalue,10);tmpvalue=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						if(tmpvalue<=0 || tmpvalue>10){
							cout<<"PERIOD NOT SET. Time period must be >0 and <=10 seconds\r\n"<<endl;
							}
						else{
							period=tmpvalue;
							h_err(ioctl(dev,BNS_SET_PERIODS,(int)(period/164.4e-6))); //conversion factor necessary
							}
						break;
					case PERIOD: /* exactly same as above (TIME_PERIOD), here for human compatibility */
						cout<<"time period.\r\nCurrent time period: "<<period<<" seconds. Enter new time period: "<<flush;
						echo();
						getnstr(inptvalue,10);tmpvalue=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						if(tmpvalue<=0 || tmpvalue>10){
							cout<<"PERIOD NOT SET. Time period must be >0 and <=10 seconds\r\n"<<endl;
							}
						else{
							period=tmpvalue;
							h_err(ioctl(dev,BNS_SET_PERIODS,(int)(period/164.4e-6))); //conversion factor necessary
							}
						break;
					case MS_RATIO:
						cout<<"mark:space ratio.\r\nCurrent mark:space ratio: "<<msr<<" Enter new mark:space ratio: "<<flush;
						echo();
						getnstr(inptvalue,10);msr=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						update_slm=TRUE;
						break;
					default:
						cout<<"Nothing changed.\n";
						cout<<"\r"<<endl;
						break;
					}
				update_slm=TRUE;
				break;

			case FACTOR:
				cout<<"Set factor: "<<flush;
				kp2=getchar();
				switch(kp2){
					case LINE_SPACING: //kept for backwards 'human compatability'
						cout<<"w.\r\nCurrent w factor: "<<dw*1e6<<" microns. Enter new w factor: "<<flush;
						echo();
						getnstr(inptvalue,10);dw=strtod(inptvalue,NULL)/1e6;
						noecho();
						cout<<"\r"<<flush;
						break;
					case WAVELENGTH: //exactly the same thing as above, just a different keypress
						cout<<"w.\r\nCurrent w factor: "<<dw*1e6<<" microns. Enter new w factor: "<<flush;
						echo();
						getnstr(inptvalue,10);dw=strtod(inptvalue,NULL)/1e6;
						noecho();
						cout<<"\r"<<flush;
						break;
					case FOCAL_LENGTH:
						cout<<"focal length.\r\nCurrent mean focal length factor: "<<df*1e6<<" microns. Enter new mean focal length factor: "<<flush;
						echo();
						getnstr(inptvalue,10);df=strtod(inptvalue,NULL)/1e6;
						cout<<"\r"<<flush;
						noecho();
						break;
					case X_OFFSET:
						cout<<"x offset.\r\nCurrent x offset factor: "<<dx*1e6<<" microns. Enter new x offset factor: "<<flush;
						echo();
						getnstr(inptvalue,10);dx=strtod(inptvalue,NULL)/1e6;
						cout<<"\r"<<flush;
						noecho();
						break;
					default:
						cout<<"Nothing changed.\n";
						cout<<"\r"<<endl;
						break;
					}
				break;

			case QUIT:
				cout<<"Press 'p' to keep slm power on after quit,\n\r";
				cout<<"press 'n' to not quit,\n\r";
				cout<<"press any other key to power off slm and quit..."<<flush;
				echo();
				kp2=getchar();
				noecho();
				switch(kp2){
					case 'n':
						cout<<"Quit cancelled\r\n"<<endl;
						break;
					case 'p':
						nocbreak(); echo(); endwin();
						cout<<"Quitting, leaving SLM power on\n"<<endl;
						quit=TRUE;
						break;
					default:
						nocbreak(); echo(); endwin();
						cout<<"Quitting, turning off SLM power\n"<<endl;
						quit=TRUE;
						h_err(ioctl(dev,BNS_POWER_OFF,dummy));
						break;
					}
				break;
			case ATTRIBUTES:
				if(first_run==TRUE){
					cout<<"Cannot determine attributes, since they have not been set yet.\r"<<endl;
					cout<<"Press `d' to set them to defaults.\r"<<endl<<endl;
					}
				else{
					cout<<"The arcs have the following characteristics:\r"<<endl;
					cout<<"Wavelength:               "<<w*1e6<<" microns\r"<<endl;
					cout<<"Mean focal length:        "<<f*1e6<<" microns\r"<<endl;
					cout<<"x offset:                 "<<xoff*1e6<<" microns\r"<<endl;
					cout<<"Mark:space ratio:         "<<msr<<"\r"<<endl<<endl;
					cout<<"Current scale factors:\r"<<endl;
					cout<<"Wavelength factor:        "<<dw*1e6<<" microns\r"<<endl;
					cout<<"Mean focal length factor: "<<df*1e6<<" microns\r"<<endl;
					cout<<"x offset factor:          "<<dx*1e6<<" microns\r"<<endl<<endl;
					cout<<"SLM image size:           "<<slm_height*1e6<<" microns\r"<<endl;
					cout<<"Distortion factor:        "<<distortion_factor<<"\r"<<endl;
					cout<<"Time period:              "<<period<<"\r"<<endl;
					cout<<"Laser A amp:              "<<lasera_amp<<"\r"<<endl;
					cout<<"Laser B amp:              "<<laserb_amp<<"\r"<<endl;
					cout<<"Laser A time:             "<<lasera_time<<"\r"<<endl;
					cout<<"Laser B time:             "<<laserb_time<<"\r"<<endl;
					cout<<"The power is:             ";
					if(power_on==TRUE) cout<<"ON\r"<<endl<<endl; else cout<<"OFF\r"<<endl<<endl;
					}
				break;
			case HELP:
				cout<<endl<<"Use these keys to adjust the arc attributes by the current scale factors:\r"<<endl;
				cout<<"   + -   increase/decrease w\r"<<endl;
				cout<<"   v ^   increase/decrease mean focal length\r"<<endl;
				cout<<"   < >   move x offset to the left/right\r"<<endl<<endl;
				cout<<"To set the values absolutely, press `s' followed by:\r"<<endl;
				cout<<"    w    w\r"<<endl;
				cout<<"    f    mean focal length\r"<<endl;
				cout<<"    x    x offset\r"<<endl;
				cout<<"    s    SLM size\r"<<endl;
//				cout<<"    d    distortion factor\r"<<endl;
				cout<<"    m    mark:space ratio (actually the portion of light that is 'off')\r"<<endl;
				cout<<"    t    time period\r"<<endl;
				cout<<"    a    Laser A amp\r"<<endl;
				cout<<"    A    Laser A time (NOTE CAPITAL \"A\")\r"<<endl;
				cout<<"    b    Laser B amp\r"<<endl;
				cout<<"    B    Laser B time (NOTE CAPITAL \"B\")\r"<<endl;
				cout<<"                     ... and then enter the value\r"<<endl<<endl;
				cout<<"To change the scale factor, press `f' followed by:\r"<<endl;
				cout<<"    w    w factor\r"<<endl;
				cout<<"    f    mean focal length factor\r"<<endl;
				cout<<"    x    x offset factor\r"<<endl;
				cout<<"                     ... and then enter the value\r"<<endl<<endl;
				cout<<"    1    turn SLM power ON\r"<<endl;
				cout<<"    2    turn SLM power OFF\r"<<endl;
				cout<<"Press `q' to quit, `a' to show the arc attributes, `d' for defaults.\r"<<endl;
				break;
			case DEFAULT:
				cout<<"Arc attributes, scales factors etc have been reset to default.\r"<<endl;
				cout<<"Press `a' to find out what they are now.\r"<<endl<<endl;

				xoff=0;
				f=2e-3;
				w=36.9e-6;
				slm_height=2.42e-3;
				distortion_factor=1;
				msr=0.5;
				period=2;
				dx=100e-6;
				df=100e-6;
				dw=1e-6;
				lasera_amp=255;
				laserb_amp=255;
				lasera_time=4;
				laserb_time=4;

				update_slm=TRUE;
				break;
			case RETURN:
				cout<<"\r"<<endl;
				break;
			default:
				break;
			}


		// now work out the conversion factors from real world units into SLM pixels

		py=slm_height/ROW_SIZE; //in metres
		px=py/distortion_factor;

// Internally the compressed direction is called X and the uncompressed direction is called
// Y.  This works lovely BUT as of the physical set up of june 2001 the horizontal (distorted)
// direction is Y and the vertical (undistorted) direction is X
		if(update_slm==TRUE){
			// note that the coordinates are transposed here to match the
			// physical layout of the slm as of june 2001 matt
			// the logical slm X direction corresponds to vertical 
			// on the slm with +ve X pointing down
			// the logical Y corresponds to horizontal and is the squished aspect
			// postive Ys correspond to movement in the right hand direction
			// as determined by the camera
			SLM_upload_arcs(dev,-xoff,f,py,px,w,msr);
			update_slm=FALSE;
			first_run=FALSE;
			}		
		}while(quit!=TRUE);
	cout<<"Saving defaults ....\n";
		
	save_defaults(w, f, xoff, msr, dw, df, dx, slm_height, distortion_factor, period, lasera_amp, laserb_amp, lasera_time, laserb_time);
	}


// Internally the compressed direction is called X and the uncompressed direction is called
// Y.  This works lovely BUT as of the physical set up of june 2001 the horizontal (distorted)
// direction is Y and the vertical (undistorted) direction is X



void set_lasera(int dev, unsigned int t_period, unsigned int amplitude){
	div_t l_div;

	l_div=div((int)t_period,16);
	h_err(ioctl(dev,BNS_SET_LASERA,(unsigned int)(65536*l_div.quot)+(unsigned int)(256*l_div.rem)+(unsigned int)amplitude));
	}

void set_laserb(int dev, unsigned int t_period, unsigned int amplitude){
	div_t l_div;

	l_div=div((int)t_period,16);
	h_err(ioctl(dev,BNS_SET_LASERB,(unsigned int)(65536*l_div.quot)+(unsigned int)(256*l_div.rem)+(unsigned int)amplitude));
	}


void SLM_upload_arcs(int dev,float xoff,float yoff,float px, float py,float w,float msr){

int pos, by;
int offset=0;
int err;
int * dummy=NULL;

int x,y;
float xc,yc,yc2;

//unsigned char frame[8192];
float dist_psf[SIZE];
unsigned char arcs[SIZE];
FILE *t;
int ap; //ap- arc pointer

//cout<<"X offset="<<xoff<<"Y offset ="<<yoff<<" px="<<px<<"py="<<py<<"\n";

// calulated the distance cone to the focal spot for each pixel
// x and y are the logical coords to the arrays, xc and yc are the distances from focus to pixel at (x,y)
//cout<<"px "<<px<<" py "<<py<<"\n";
	for(y=0;y<ROW_SIZE;y++){
		yc=((float)y-(ROW_SIZE/2))*py+yoff;
		yc2=yc*yc;// compiler should remove extra variable, leave for clarity
		for(x=0;x<ROW_SIZE;x++){
			xc=((float)x-(ROW_SIZE/2))*px+xoff;
			dist_psf[x+(y*ROW_SIZE)]=sqrt((xc*xc)+yc2);
			}
		}

//now convert it to 0's and 255's
	for(ap=0;ap<SIZE-1;ap++){
			arcs[ap]=(char)((fmod(dist_psf[ap],w) >= (w*msr))*255);
			}


//	t=fopen("dist_psf.dat","w");
//	fwrite((void *)dist_psf,sizeof(float),65536,t);
//	fclose(t);
//
//	t=fopen("arcs.dat","w");
//	fwrite((void *)arcs,sizeof(char),65536,t);
//	fclose(t);
//
//	t=fopen("coded.dat","w");
//	fwrite((void *)frame,sizeof(char),8192,t);
//	fclose(t);


//	cout<<"Enabling write\r"<<endl;
	h_err(ioctl(dev,BNS_ENABLE_WRITE,dummy));

//	cout<<"Uploading to frame 0\r"<<endl;
	h_err(ioctl(dev,BNS_UPLOAD_IMAGE,0));
	err=ROW_SIZE;
	for(pos=offset,by=0;by<512;by++,pos+=IMAGE_LENGTH){ 
		err=write(dev,arcs+pos,ROW_SIZE);
		if(err!=ROW_SIZE){
			cout<<"WRITE ERROR (writes enabled?) err="<<err<<"\r"<<endl;
			}
		}

//	cout<<"Disabling write/allowing image\r"<<endl;
	h_err(ioctl(dev,BNS_DISABLE_WRITE,dummy));

//	cout<<"Displaying image\r\n"<<endl;
	h_err(ioctl(dev,BNS_DISPLAY_IMAGE,0));

	}



// distrotion_factor is really y_distort_sq and from the maths it is the linear stretch factor ^2 
// the array index sequence is unconventional and possibly slow
// the use of pow() rather than << is terribly slow
// y_offset is in the propagation direction (focal length)
//
// the z-axis or propagaton direction axis is the y-axis or slowly varying axis or large step axis or uncompressed axis
//
// nasty mixture of mm and microns, better off all in metres
// all SLM_ units are in pixels
//

void h_err(int err){
        if(err!=0){
		nocbreak(); echo(); endwin();
                cout<<"Got error "<<err<<endl;

                exit(1);
                }
        usleep(1000);
        }


/* sanity extension by matt august 2002 */
void save_defaults(float w, float f, float x, float msr, float dw, float df, float dx, float h, float dd, float t, unsigned int lasera_amp, unsigned int laserb_amp, unsigned int lasera_time, unsigned int laserb_time){
	
ofstream os;
	os.open(DEFAULTS_FILENAME);
	os<<w<<"\n";
	os<<f<<"\n";
	os<<x<<"\n";
	os<<msr<<"\n";
	os<<dw<<"\n";
	os<<df<<"\n";
	os<<dx<<"\n";
	os<<h<<"\n";
	os<<dd<<"\n";
	os<<t<<"\n";
	os<<lasera_amp<<"\n";
	os<<laserb_amp<<"\n";
	os<<lasera_time<<"\n";
	os<<laserb_time<<"\n";
	os.close();
	}
/* sanity extension by matt august 2002 */
int load_defaults(float *w, float *f, float *x, float *msr, float *dw, float *df, float *dx, float *h, float *dd, float *t, unsigned int *lasera_amp, unsigned int *laserb_amp, unsigned int *lasera_time, unsigned int *laserb_time){
	
ifstream is;

	if(access(DEFAULTS_FILENAME,R_OK)!=0){
		cout<<"*** Can't read the last settings from "<<DEFAULTS_FILENAME<<"\n";
		return TRUE;
		}
	is.open(DEFAULTS_FILENAME);
	is>>*w;
	is>>*f;
	is>>*x;
	is>>*msr;
	is>>*dw;
	is>>*df;
	is>>*dx;
	is>>*h;
	is>>*dd;
	is>>*t;
	is>>*lasera_amp;
	is>>*laserb_amp;
	is>>*lasera_time;
	is>>*laserb_time;
	is.close();
	return FALSE;
	}
