/* set_bns adapted from "set_slm..c" (for displaytech slm) and "manual_bns.cc" (for bns slm) */
/* sds 11th Feb 2002 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include "../../driver/slm.h"

#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <curses.h>
#include <stdlib.h>
#include <iostream>
using namespace std;
#include <string.h>

#define SLM "/dev/slm"
#define NFRAMES 1024
#define ROW_SIZE (512)
#define IMAGE_LENGTH 512
#define SIZE 512*IMAGE_LENGTH
#define N 100000
#define BOOL    int
#define TRUE    1
#define FALSE   0

void h_err(int);
void SLM_upload_arcs(int,float,float,float,float,float,float);
BOOL sc(char *,char *);

main(int argc,char ** argv){

// set initial values

int*	dummy=NULL;
int	dev;
unsigned char	image[SIZE];
int n=0,err;

float xoff=0;
float focal_length=2e-3;
float wavelength=36.6e-6;	// steel
//float slm_height=2.45e-3;	// pre July 2002
//float slm_height=2.6e-3;	// July 2002 -> September 2002
float slm_height=2.42e-3;	// september 2002 ->
float distortion_factor=1;
float msr=0.5;
float period=0.015; //in seconds;

float tmpvalue;
int power_on=TRUE;
int verbose_mode=FALSE;

int index=1;

float px,py;

        if(argc==1){
                cout<<"Need some arguments!"<<endl<<endl;
                cout<<"-x    -xoff            : x offset (in metres)      default "<<xoff<<endl;
                cout<<"-f    -focal_length    : focal length (in metres)  default "<<focal_length<<endl;
                cout<<"-l    -wavelength      : line spacing (in metres)  default "<<wavelength<<endl;
                cout<<"-t    -time_period     : time period (in seconds)  default "<<period<<endl;
                cout<<"-s    -slm_height      : slm height (in metres)    default "<<slm_height<<endl;
                cout<<"-d    -distortion      : distortion factor         default "<<distortion_factor<<endl;
		cout<<"-n    -no_power        : do NOT turn on SLM power  (not default)"<<endl;
		cout<<"-v    -verbose         : confirm all settings      (not default)"<<endl;
                exit(3);
                }
        while(index<argc){
                if(sc(argv[index],"-v")||sc(argv[index],"-verbose")){
			verbose_mode=TRUE;
			}

                if(sc(argv[index],"-n")||sc(argv[index],"-no_power")){
			power_on=FALSE;
			}

                if(sc(argv[index],"-x")||sc(argv[index],"-xoff")){
                        sscanf(argv[++index],"%f",&xoff);
                        }
 
                if(sc(argv[index],"-f")||sc(argv[index],"-focal_length")){
                        sscanf(argv[++index],"%f",&focal_length);
                        }
 
                if(sc(argv[index],"-l")||sc(argv[index],"-wavelength")){
                        sscanf(argv[++index],"%f",&wavelength);
                        }
 
                if(sc(argv[index],"-t")||sc(argv[index],"-time_period")){
                        sscanf(argv[++index],"%f",&period);
                        }
 
                if(sc(argv[index],"-s")||sc(argv[index],"-slm_height")){
                        sscanf(argv[++index],"%f",&slm_height);
                        }
 
                if(sc(argv[index],"-d")||sc(argv[index],"-distortion")){
                        sscanf(argv[++index],"%f",&distortion_factor);
                        }
                index++;
                }


	// initialisation...
        dev=open(SLM,O_RDWR);
        if(dev<0){
		cout<<"Failed to open "<<SLM<<endl;
                perror("slm_test:FTO:");
                exit(1);
                }

%ioctl(dev,BNS_SET_DPEN_DELAY,8000);
%ioctl(dev,BNS_DPEN_MODE_SLEEP,NULL);

	if(verbose_mode==TRUE){
	        cout<<"Setting SLM arc attributes:\n"<<flush;
	        cout<<"Wavelength:             "<<wavelength*1e6<<" microns"<<endl;
	        cout<<"Mean focal length:      "<<focal_length*1e6<<" microns"<<endl;
	        cout<<"x offset:               "<<xoff*1e6<<" microns"<<endl<<endl;
	        cout<<"SLM image height:       "<<slm_height*1e6<<" microns"<<endl;
	        cout<<"Distortion factor:      "<<distortion_factor<<endl;
	        cout<<"Time period:            "<<period<<endl;
		cout<<"The power is ";
		if(power_on==TRUE){
			cout<<"being set: ON"<<endl;
			}
		else cout<<"NOT being set"<<endl;
		}

	if(power_on==TRUE){
		h_err(ioctl(dev,BNS_POWER_ON,dummy));
		}

        h_err(ioctl(dev,BNS_SET_PERIODS,(int)(period/164.4e-6)));

	// now work out the conversion factors from real world units into SLM pixels

	py=slm_height/ROW_SIZE; //in metres
	px=py/distortion_factor;

	// note that the coordinates are transposed here to match the
	// physical layout of the slm as of june 2001 matt
	// the logical slm X direction corresponds to vertical 
	// on the slm with +ve X pointing down
	// the logical Y corresponds to horizontal and is the squished aspect
	// postive Ys correspond to movement in the right hand direction
	// as determined by the camera
	SLM_upload_arcs(dev,-xoff,focal_length,py,px,wavelength,msr);
	}


// Internally the compressed direction is called X and the uncompressed direction is called
// Y.  This works lovely BUT as of the physical set up of june 2001 the horizontal (distorted)
// direction is Y and the vertical (undistorted) direction is X

void SLM_upload_arcs(int dev,float xoff,float yoff,float px, float py,float wavelength,float msr){

int pos, by;
int offset=0;
int err;
int * dummy=NULL;

int x,y;
float xc,yc,yc2;

//unsigned char frame[8192];
float dist_psf[SIZE];
unsigned char arcs[SIZE];
FILE *t;
int ap; //ap- arc pointer

//cout<<"X offset="<<xoff<<"Y offset ="<<yoff<<" px="<<px<<"py="<<py<<"\n";

// calulated the distance cone to the focal spot for each pixel
// x and y are the logical coords to the arrays, xc and yc are the distances from focus to pixel at (x,y)
//cout<<"px "<<px<<" py "<<py<<"\n";
	for(y=0;y<ROW_SIZE;y++){
		yc=((float)y-(ROW_SIZE/2))*py+yoff;
		yc2=yc*yc;// compiler should remove extra variable, leave for clarity
		for(x=0;x<ROW_SIZE;x++){
			xc=((float)x-(ROW_SIZE/2))*px+xoff;
			dist_psf[x+(y*ROW_SIZE)]=sqrt((xc*xc)+yc2);
			}
		}

//now convert it to 0's and 255's
	for(ap=0;ap<SIZE-1;ap++){
			arcs[ap]=(char)((fmod(dist_psf[ap],wavelength) >= (wavelength*msr))*255);
			}


//	t=fopen("dist_psf.dat","w");
//	fwrite((void *)dist_psf,sizeof(float),65536,t);
//	fclose(t);
//
//	t=fopen("arcs.dat","w");
//	fwrite((void *)arcs,sizeof(char),65536,t);
//	fclose(t);
//
//	t=fopen("coded.dat","w");
//	fwrite((void *)frame,sizeof(char),8192,t);
//	fclose(t);


//	cout<<"Enabling write\r"<<endl;
	h_err(ioctl(dev,BNS_ENABLE_WRITE,dummy));

//	cout<<"Uploading to frame 0\r"<<endl;
	h_err(ioctl(dev,BNS_UPLOAD_IMAGE,0));
	err=ROW_SIZE;
	for(pos=offset,by=0;by<512;by++,pos+=IMAGE_LENGTH){ 
		err=write(dev,arcs+pos,ROW_SIZE);
		if(err!=ROW_SIZE){
			cout<<"WRITE ERROR (writes enabled?) err="<<err<<"\r"<<endl;
			}
		}
%usleep(3000);

//	cout<<"Disabling write/allowing image\r"<<endl;
	h_err(ioctl(dev,BNS_DISABLE_WRITE,dummy));
%usleep(9000);

//	cout<<"Displaying image\r\n"<<endl;
	h_err(ioctl(dev,BNS_DISPLAY_IMAGE,0));
%usleep(3000);

	}


// distrotion_factor is really y_distort_sq and from the maths it is the linear stretch factor ^2 
// the array index sequence is unconventional and possibly slow
// the use of pow() rather than << is terribly slow
// y_offset is in the propagation direction (focal length)
//
// the z-axis or propagaton direction axis is the y-axis or slowly varying axis or large step axis or uncompressed axis
//
// nasty mixture of mm and microns, better off all in metres
// all SLM_ units are in pixels
//

void h_err(int err){
        if(err!=0){
                cout<<"Got error "<<err<<endl;

                exit(1);
                }
        usleep(1000);
        }


BOOL    sc(char *con,char *var){
        if(strcmp(con,var)==0){
                return TRUE;
                }
        return FALSE;
        }

